@@ -40,15 +40,15 @@ static void stop_drawing(void);
         Cells
 */
 
-/* Msec of no mouse motion before a cell is finished */
-#define MOTION_TIMEOUT 1000
-
 #define ALTERNATES 5
 #define CELL_BASELINE (cell_height / 3)
 #define CELL_BORDER (cell_height / 12)
 #define KEY_WIDGET_COLS 4
 #define KEY_WIDGET_BORDER 6
 
+/* Msec of no mouse motion before a cell is finished */
+#define MOTION_TIMEOUT 500
+
 /* Cell flags */
 #define CELL_SHOW_INK   0x01
 #define CELL_DIRTY      0x02
@@ -631,7 +631,7 @@ static void wrap_cells(int new_rows, int new_cols)
                        new_cols * sizeof (Cell));
         }
 
-        /* Only free the cell array, NOT the sample as we have copied the
+        /* Only free the cell array, NOT the samples as we have copied the
            Sample data over to the new cell array */
         g_free(cells);
         cells = new_cells;
@@ -795,6 +795,7 @@ static gboolean finish_timeout(void)
 {
         finish_cell(current_cell);
         render_dirty();
+        timeout_source = 0;
         start_timeout();
         return FALSE;
 }
@@ -804,6 +805,7 @@ static gboolean row_timeout(void)
 {
         pack_cells(cell_rows + 1, cell_cols);
         cell_widget_render();
+        timeout_source = 0;
         return FALSE;
 }
 
@@ -824,6 +826,7 @@ static int check_clear(void)
 static gboolean is_clear_timeout(void)
 /* Motion timeout for checking clear state */
 {
+        timeout_source = 0;
         if (is_clear || !check_clear())
                 return FALSE;
 
@@ -844,6 +847,7 @@ static gboolean hold_timeout(void)
                 stop_drawing();
                 show_context_menu(1, gtk_get_current_event_time());
         }
+        timeout_source = 0;
         return FALSE;
 }
 
@@ -853,14 +857,12 @@ static void start_timeout(void)
 {
         GSourceFunc func = NULL;
 
+        if (potential_hold)
+                return;
         stop_timeout();
         if (cross_out)
                 return;
 
-        /* Pick a timeout event to trigger */
-        if (potential_hold)
-                func = (GSourceFunc)hold_timeout;
-
         /* Events below are not triggered while drawing */
         if (!drawing) {
                 if (input)
@@ -876,6 +878,15 @@ static void start_timeout(void)
                 timeout_source = g_timeout_add(MOTION_TIMEOUT, func, NULL);
 }
 
+static void start_hold(void)
+{
+        potential_hold = TRUE;
+        if (timeout_source)
+                g_source_remove(timeout_source);
+        timeout_source = g_timeout_add(MOTION_TIMEOUT,
+                                       (GSourceFunc)hold_timeout, NULL);
+}
+
 void cell_widget_set_cursor(int recreate)
 /* Set the drawing area cursor to a black box pen cursor or to a blank cursor
    depending on which is appropriate */
@@ -1383,8 +1394,7 @@ static gboolean button_press_event(GtkWidget *widget, GdkEventButton *event)
                 if (inserting)
                         potential_insert = TRUE;
                 else if (cells[current_cell].ch) {
-                        potential_hold = TRUE;
-                        start_timeout();
+                        start_hold();
                 } else
                         draw(event->x, event->y);
 
@@ -1546,7 +1556,6 @@ static void configure_keys(void)
 static gboolean configure_event(void)
 /* Create a new backing pixmap of the appropriate size */
 {
-        PangoCairoFontMap *font_map;
         int new_cols;
 
         /* Do nothing if we are not visible */
@@ -1583,13 +1592,6 @@ static gboolean configure_event(void)
                 cairo_destroy(cairo);
         cairo = gdk_cairo_create(GDK_DRAWABLE(pixmap));
 
-        /* Pango context */
-        if (pango)
-                g_object_unref(pango);
-        font_map = PANGO_CAIRO_FONT_MAP(pango_cairo_font_map_new());
-        pango = pango_cairo_font_map_create_context(font_map);
-        g_object_unref(font_map);
-
         /* Set font size */
         pango_font_description_set_absolute_size(pango_font_desc, PANGO_SCALE *
                                                  (cell_height -
@@ -1712,17 +1714,11 @@ int cell_widget_update_colors(void)
         color_inactive = custom_inactive_color;
         color_ink = custom_ink_color;
         color_select = custom_select_color;
-#if GTK_CHECK_VERSION(2, 10, 0)
         if (style_colors) {
-                gtk_style_lookup_color(window->style, "base_color",
-                                       &color_active);
-                gtk_style_lookup_color(window->style, "text_color",
-                                       &color_ink);
-                gtk_style_lookup_color(window->style, "bg_color",
-                                       &color_inactive);
-                shade_gdk_color(&color_inactive, &color_inactive, 0.95);
+                color_active = window->style->base[0];
+                color_ink = window->style->text[0];
+                color_inactive = window->style->bg[1];
         }
-#endif
         return !gdk_colors_equal(&old_active, &color_active) ||
                !gdk_colors_equal(&old_inactive, &color_inactive) ||
                !gdk_colors_equal(&old_ink, &color_ink) ||
@@ -1884,6 +1880,9 @@ int cell_widget_insert(void)
                 return FALSE;
         chars = 0;
 
+        /* Prepare for sending key events */
+        key_event_update_mappings();
+
         /* Need to send the keys out in reverse order for right_to_left mode
            because the cells are displayed with columns reversed */
         if (right_to_left)
@@ -2041,7 +2040,9 @@ int cell_widget_get_height(void)
 }
 
 GtkWidget *cell_widget_new(void)
+/* Creates the Cell widget. Should only be called once per program run! */
 {
+        PangoCairoFontMap *font_map;
         GtkWidget *widget, *hbox;
 
         /* Initial settings */
@@ -2112,6 +2113,11 @@ GtkWidget *cell_widget_new(void)
         pango_font_desc = pango_font_description_new();
         pango_font_description_set_family(pango_font_desc, "Monospace");
 
+        /* Pango context */
+        font_map = PANGO_CAIRO_FONT_MAP(pango_cairo_font_map_new());
+        pango = pango_cairo_font_map_create_context(font_map);
+        g_object_unref(font_map);
+
         /* Clear cells */
         cell_widget_clear();
 
@@ -2127,5 +2133,16 @@ GtkWidget *cell_widget_new(void)
 void cell_widget_cleanup(void)
 {
         key_widget_cleanup(key_widget);
+
+        /* Freeing memory when closing is important when trying to sort
+           legitimate memory leaks from left-over memory */
+        if (pixmap)
+                g_object_unref(pixmap);
+        if (pixmap_gc)
+                g_object_unref(pixmap_gc);
+        if (cairo)
+                cairo_destroy(cairo);
+        if (pango)
+                g_object_unref(pango);
 }
 
