@@ -319,15 +319,11 @@ int key_widget_update_colors(void)
         color_text.red = 255 * COLOR_SCALE;
         color_text.green = 255 * COLOR_SCALE;
         color_text.blue = 255 * COLOR_SCALE;
-#if GTK_CHECK_VERSION(2, 10, 0)
-        gtk_style_lookup_color(window->style, "bg_color", &color_bg);
+        color_bg = window->style->bg[0];
         if (style_colors) {
-                gtk_style_lookup_color(window->style, "selected_bg_color",
-                                       &color_keys);
-                gtk_style_lookup_color(window->style, "selected_fg_color",
-                                       &color_text);
+                color_keys = window->style->dark[3];
+                color_text = window->style->fg[3];
         }
-#endif
         shade_gdk_color(&color_keys, &color_keys_dark, 0.67);
         shade_gdk_color(&color_keys, &color_keys_on, 0.75);
         return !gdk_colors_equal(&old_keys, &color_keys) ||
@@ -547,6 +543,9 @@ gboolean key_widget_button_press(GtkWidget *widget, GdkEventButton *event,
                 if (!(key->flags & KEY_SHIFT))
                         press_sticky_keys(key_widget, TRUE);
 
+                /* Prepare for sending key events */
+                key_event_update_mappings();
+
                 /* Press down the active key's keysym */
                 keysym = key->keysym;
                 if (is_shifted(key))
