@@ -25,6 +25,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
+#include <stdio.h>
 #include <errno.h>
 #ifdef HAVE_GNOME
 #include <libgnome/libgnome.h>
@@ -240,7 +241,7 @@ int profile_read_only, keyboard_only = FALSE;
 
 static GIOChannel *channel;
 static char profile_buf[4096], *profile_end = NULL, profile_swap,
-            *force_profile = NULL;
+            *force_profile = NULL, *profile_tmp = NULL;
 static int force_read_only;
 
 static int is_space(int ch)
@@ -281,17 +282,13 @@ static void create_user_dir(void)
         g_free(path);
 }
 
-int profile_open(int read_only)
+static int profile_open_read(void)
 /* Open the profile file for reading. Returns TRUE if the profile was opened
    successfully. */
 {
         char *path;
 
-        if (!read_only && force_read_only) {
-                g_debug("Not saving profile, opened in read-only mode");
-                return FALSE;
-        }
-        profile_read_only = read_only;
+        profile_read_only = TRUE;
 
         /* Try opening a command-line specified profile first */
         if (force_profile &&
@@ -306,12 +303,6 @@ int profile_open(int read_only)
         }
         g_free(path);
 
-        /* Don't write to system profile */
-        if (!read_only) {
-                g_debug("Refusing to save to system profile");
-                return FALSE;
-        }
-
         /* Open system profile */
         path = g_build_filename(PKGDATADIR, "profile", NULL);
         if (profile_open_channel("system", path)) {
@@ -323,10 +314,157 @@ int profile_open(int read_only)
         return FALSE;
 }
 
-void profile_close(void)
+static int profile_open_write(void)
+/* Open a temporary profile file for writing. Returns TRUE if the profile was
+   opened successfully. */
 {
-        if (channel)
-                g_io_channel_unref(channel);
+        GError *error;
+        gint fd;
+
+        if (force_read_only) {
+                g_debug("Not saving profile, opened in read-only mode");
+                return FALSE;
+        }
+        profile_read_only = FALSE;
+
+        /* Open a temporary file as a channel */
+        error = NULL;
+        fd = g_file_open_tmp(PACKAGE ".XXXXXX", &profile_tmp, &error);
+        if (error) {
+                g_warning("Failed to open tmp file while saving "
+                          "profile: %s", error->message);
+                return FALSE;
+        }
+        channel = g_io_channel_unix_new(fd);
+        if (!channel) {
+                g_warning("Failed to create channel from temporary file");
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
+static int move_file(char *from, char *to)
+/* The standard library rename() cannot move across filesystems so we need a
+   function that can emulate that. This function will copy a file, byte-by-byte
+   but is not as safe as rename(). */
+{
+        GError *error = NULL;
+        GIOChannel *src_channel, *dest_channel;
+        gchar buffer[4096];
+
+        /* Open source file for reading */
+        src_channel = g_io_channel_new_file(from, "r", &error);
+        if (error) {
+                g_warning("move_file() failed to open src '%s': %s",
+                          from, error->message);
+                return FALSE;
+        }
+
+        /* Open destination file for writing */
+        dest_channel = g_io_channel_new_file(to, "w", &error);
+        if (error) {
+                g_warning("move_file() failed to open dest '%s': %s",
+                          to, error->message);
+                g_io_channel_unref(src_channel);
+                return FALSE;
+        }
+
+        /* Copy data in blocks */
+        for (;;) {
+                gsize bytes_read, bytes_written;
+
+                /* Read a block in */
+                g_io_channel_read_chars(src_channel, buffer, sizeof (buffer),
+                                        &bytes_read, &error);
+                if (bytes_read < 1 || error)
+                        break;
+
+                /* Write the block out */
+                g_io_channel_write_chars(dest_channel, buffer, bytes_read,
+                                         &bytes_written, &error);
+                if (bytes_written < bytes_read || error) {
+                        g_warning("move_file() error writing to '%s': %s",
+                                  to, error->message);
+                        g_io_channel_unref(src_channel);
+                        g_io_channel_unref(dest_channel);
+                        return FALSE;
+                }
+        }
+
+        /* Close channels */
+        g_io_channel_unref(src_channel);
+        g_io_channel_unref(dest_channel);
+
+        g_debug("move_file() copied '%s' to '%s'", from, to);
+
+        /* Should be safe to delete the old file now */
+        if (remove(from))
+                log_errno("move_file() failed to delete src");
+
+        return TRUE;
+}
+
+static int profile_close(void)
+/* Close the currently open profile and, if we just wrote the profile to a
+   temporary file, move it in place of the old profile */
+{
+        char *path = NULL;
+
+        if (!channel)
+                return FALSE;
+        g_io_channel_unref(channel);
+
+        if (!profile_tmp || profile_read_only)
+                return TRUE;
+
+        /* For some bizarre reason we may not have managed to create the
+           temporary file */
+        if (!g_file_test(profile_tmp, G_FILE_TEST_EXISTS)) {
+                g_warning("Tmp profile '%s' does not exist", profile_tmp);
+                return FALSE;
+        }
+
+        /* Use command-line specified profile path first then the user's
+           home directory profile */
+        path = force_profile;
+        if (!path)
+                path = g_build_filename(g_get_home_dir(),
+                                        "." PACKAGE, "profile", NULL);
+
+        if (g_file_test(path, G_FILE_TEST_EXISTS)) {
+                g_message("Replacing '%s' with '%s'", path, profile_tmp);
+
+                /* Don't write over non-regular files */
+                if (!g_file_test(path, G_FILE_TEST_IS_REGULAR)) {
+                        g_warning("Old profile '%s' is not a regular file",
+                                  path);
+                        goto error_recovery;
+                }
+
+                /* Remove the old profile */
+                if (remove(path)) {
+                        log_errno("Failed to delete old profile");
+                        goto error_recovery;
+                }
+        }
+        else
+                g_message("Creating new profile '%s'", path);
+
+        /* Move the temporary profile file in place of the old one */
+        if (rename(profile_tmp, path)) {
+                log_errno("rename() failed to move tmp profile in place");
+                if (!move_file(profile_tmp, path))
+                        goto error_recovery;
+        }
+
+        if (path != force_profile)
+                g_free(path);
+        return TRUE;
+
+error_recovery:
+        g_warning("Recover tmp profile at '%s'", profile_tmp);
+        return FALSE;
 }
 
 const char *profile_read(void)
@@ -476,7 +614,7 @@ void version_read(void)
 
 #define NUM_PROFILE_CMDS (sizeof (profile_cmds) / sizeof (*profile_cmds))
 
-int profile_line, log_level = 3;
+int profile_line, log_level = 4;
 
 static char *log_filename = NULL;
 static FILE *log_file = NULL;
@@ -501,7 +639,7 @@ static GOptionEntry command_line_opts[] = {
         { "log-level", 0, 0, G_OPTION_ARG_INT, &log_level,
           "Log threshold (0=silent, 7=debug)", "4" },
         { "log-file", 0, 0, G_OPTION_ARG_STRING, &log_filename,
-          "Log file to use instead of stdout", "cellwriter.log" },
+          "Log file to use instead of stdout", PACKAGE ".log" },
         { "xid", 0, 0, G_OPTION_ARG_NONE, &window_embedded,
           "Embed the main window (XEMBED)", NULL },
         { "show-window", 0, 0, G_OPTION_ARG_NONE, &window_force_show,
@@ -561,15 +699,15 @@ void cleanup(void)
                 single_instance_cleanup();
 
         /* Save profile */
-        if (!window_embedded && profile_open(FALSE)) {
+        if (!window_embedded && profile_open_write()) {
                 unsigned int i;
 
                 profile_write(va("version %d\n", PROFILE_VERSION));
                 for (i = 0; i < NUM_PROFILE_CMDS; i++)
                         if (profile_cmds[i].write_func)
                                 profile_cmds[i].write_func();
-                profile_close();
-                g_debug("Profile saved");
+                if (profile_close())
+                        g_debug("Profile saved");
         }
 
         /* Close log file */
@@ -602,17 +740,11 @@ static void hook_signals(void)
 }
 
 void log_print(const char *format, ...)
+/* Print to the log file or stderr */
 {
         FILE *file;
         va_list va;
 
-        /* Try to open the log-file */
-        if (log_filename && !log_file) {
-                log_file = fopen(log_filename, "w");
-                log_filename = NULL;
-        }
-
-        /* Print to file or stderr */
         file = log_file;
         if (!file) {
                 if (window_embedded)
@@ -714,7 +846,6 @@ int main(int argc, char *argv[])
         const char *token;
 
         /* Initialize GTK+ */
-        g_set_application_name(PACKAGE_NAME);
         error = NULL;
         if (!gtk_init_with_args(&argc, &argv,
                                 "grid-entry handwriting input panel",
@@ -725,6 +856,13 @@ int main(int argc, char *argv[])
         log_level = 1 << log_level;
         g_log_set_handler(NULL, -1, (GLogFunc)log_func, NULL);
 
+        /* Try to open the log-file */
+        if (log_filename) {
+                log_file = fopen(log_filename, "w");
+                if (!log_file)
+                        g_warning("Failed to open log-file '%s'", log_filename);
+        }
+
         /* See if the program is already running */
         g_message("Starting " PACKAGE_STRING);
         create_user_dir();
@@ -762,7 +900,7 @@ int main(int argc, char *argv[])
         recognize_init();
 
         /* Read profile */
-        if (profile_open(TRUE)) {
+        if (profile_open_read()) {
                 profile_line = 1;
                 g_message("Parsing profile");
                 do {
