@@ -251,6 +251,7 @@ static gboolean window_configure(GtkWidget *widget, GdkEventConfigure *event)
                     (height_change && window_docked == WINDOW_DOCKED_BOTTOM)) {
                         screen_width = screen_w;
                         screen_height = screen_h;
+                        trace("move-sizing bottom-docked window");
                         docked_move_resize();
                 }
                 update_struts();
@@ -318,7 +319,6 @@ void window_set_docked(int mode)
 
         /* Restore the old window position */
         if (!mode) {
-                set_geometry_hints();
                 update_struts();
                 window_frame = window_frame_saved;
                 gtk_window_move(GTK_WINDOW(window), window_frame.x,
@@ -561,8 +561,11 @@ void window_create(void)
                          G_CALLBACK(window_configure), NULL);
         gtk_window_set_accept_focus(GTK_WINDOW(window), FALSE);
         gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
-        gtk_window_set_type_hint(GTK_WINDOW(window),
-                                 GDK_WINDOW_TYPE_HINT_DOCK);
+
+        /* This hint was alleged to fix the strut problems with metacity but
+           doesn't and only causes the window to overlap the docked panels */
+        /*gtk_window_set_type_hint(GTK_WINDOW(window),
+                                 GDK_WINDOW_TYPE_HINT_DOCK);*/
 
         /* Tooltips */
         tooltips = gtk_tooltips_new();
